@@ -77,8 +77,8 @@ def create_user_gti(time, in_rate, in_drate, lognormal=True, gti_name='user_gtis
 		rate = np.log10(in_rate)
 		drate = in_drate/in_rate
 		rate_label = '$\\log_{10}(\\mathrm{rate})$'
-		level_down = np.log10(max(level_down, 1e-5))
-		level_up = np.log10(max(level_up, 1e-5))		
+		level_down = np.log10(max(in_level_down, 1e-5))
+		level_up = np.log10(max(in_level_up, 1e-5))		
 	else:
 		rate = in_rate
 		drate = in_drate
