@@ -176,7 +176,7 @@ def get_efold_frequency(nu_min, nu_max, min_en=3., max_en=20., n_bins=32, unit='
 
     return x[2]
 
-def get_nu_search_limits(filename, freq_search_interval=0.2, plot_filename=None, nyquist_factor=1, min_frequency=1e-4):
+def get_nu_search_limits(filename, freq_search_interval=0.2, plot_filename=None, nyquist_factor=1, min_frequency=1e-4, max_frequency=10.):
     """It uses the Lomb-Scargle periodogram to find a range of spin frequencies to explore
         to find a reliable periodicity
 
@@ -186,6 +186,8 @@ def get_nu_search_limits(filename, freq_search_interval=0.2, plot_filename=None,
         plot_filename (str, optional): the name of output plot file. Defaults to None to avoid plotting.
         nyquist_factor (int, optional): Lomb Scargle nymquist factor, see https://docs.astropy.org/en/stable/timeseries/lombscargle.html. Defaults to 1.
         min_frequency (float, optional): it will search peaks only above this frequency (to avoid red-noise). Defaults to 1e-4
+        max_frequency (float, optional): it will search peaks only below this frequency (to avoid red-noise). Defaults to 10
+
     Returns:
         nu_start, nu_stop: minimum and maximum frequencies to search
     """    
@@ -200,7 +202,7 @@ def get_nu_search_limits(filename, freq_search_interval=0.2, plot_filename=None,
     #power = LombScargle(lc_unbinned._times_, lc_unbinned._rates_).power(frequencies)
     frequencies, power = LombScargle(lc_unbinned._times_, lc_unbinned._rates_).autopower(nyquist_factor=nyquist_factor)
     
-    mask = frequencies > min_frequency
+    mask = (frequencies >= min_frequency) & (frequencies <= max_frequency)
     max_freq = frequencies[mask][np.argmax(power[mask])]
     nu_start = max_freq * (1.-freq_search_interval)
     nu_stop = max_freq * (1.+freq_search_interval)
@@ -229,7 +231,7 @@ none
 %s
 n
 %d
-
+%19.12e
 %19.12e
 %19.12e
 0
@@ -249,7 +251,7 @@ none
 %s
 n
 %d
-
+%19.12e
 %19.12e
 %19.12e
 0
@@ -269,7 +271,7 @@ none
 y
 %s
 %d
-
+%19.12e
 %19.12e
 %19.12e
 0
@@ -290,7 +292,7 @@ none
 y
 %s
 %d
-
+%19.12e
 %19.12e
 %19.12e
 0
@@ -306,7 +308,7 @@ y
 
 
 
-def make_enphase(freq,  min_en=3., max_en=70., en_step=0.5, n_bins=32, orbitfile=None, nudot=0):
+def make_enphase(freq,  min_en=3., max_en=70., en_step=0.5, n_bins=32, orbitfile=None, nudot=0, t_ref=0):
     
     '''
     Wrapper on timingsuite to produce Energy - Phase matrixes for source and background for both FPM units.
@@ -317,6 +319,7 @@ def make_enphase(freq,  min_en=3., max_en=70., en_step=0.5, n_bins=32, orbitfile
     :param n_bins: number of phase bins (=32)
     :param orbitfile: orbitfile for orbital correction (timingsuite) if None
     :param nudot: spinn requency derivative (=0)
+    :param t_ref: reference time for folding (if <= 0, it takes the first time)
     :return: it writes the fits files sourceA_ENPHASE.fits sourceB_ENPHASE.fits
                         backgroundA_ENPHASE.fits backgroundB_ENPHASE.fits
     '''
@@ -330,12 +333,12 @@ def make_enphase(freq,  min_en=3., max_en=70., en_step=0.5, n_bins=32, orbitfile
                 with open('timing_cmd.txt', 'w') as ff:
                     if type(en_step) == float:
                         ff.write(enphase_cmd % ('%s%s_ENPHASE.fits' % (tt, unit),
-                                         n_bins, freq, nudot, en_step, min_en, max_en) )
+                                         n_bins, t_ref, freq, nudot, en_step, min_en, max_en) )
                     else:
                         if not os.path.isfile(en_step):
                             raise FileExistsError('File %s does not exist' % en_step)
                         ff.write(enphase_cmd_binfile % ('%s%s_ENPHASE.fits' % (tt, unit),
-                                                n_bins, freq, nudot, en_step))
+                                                n_bins, t_ref, freq, nudot, en_step))
             else:
                 if not os.path.isfile(orbitfile):
                     raise FileExistsError('File %s does not exist' % orbitfile)
@@ -343,12 +346,12 @@ def make_enphase(freq,  min_en=3., max_en=70., en_step=0.5, n_bins=32, orbitfile
                 with open('timing_cmd.txt', 'w') as ff:
                     if type(en_step) == float:
                         ff.write(enphase_cmd_orbit % ('%s%s_ENPHASE.fits' % (tt, unit), orbitfile,
-                                     n_bins, freq, nudot, en_step, min_en, max_en) )
+                                     n_bins, t_ref, freq, nudot, en_step, min_en, max_en) )
                     else:
                         if not os.path.isfile(en_step):
                             raise FileExistsError('File %s does not exist' % en_step)
                         ff.write(enphase_cmd_orbit_binfile % ('%s%s_ENPHASE.fits' % (tt, unit), 
-                        orbitfile, n_bins, freq, nudot, en_step))
+                        orbitfile, n_bins, t_ref, freq, nudot, en_step))
             
             run()
             shutil.copy('%s%s_ENPHASE.fits' % (tt, unit), '%s%s_ENPHASE_%03d.fits' % (tt, unit, n_bins))
@@ -568,7 +571,7 @@ none
 %s
 n
 %d
-
+%19.12e
 %19.12e
 %19.12e
 0
@@ -587,7 +590,7 @@ none
 y
 %s
 %d
-
+%19.12e
 %19.12e
 %19.12e
 0
@@ -599,7 +602,22 @@ n
 %f
 '''
 
-def make_tphase(freq,  min_en=3., max_en=70., t_step=1000, n_bins=32, orbitfile=None, nudot=0):
+def make_tphase(freq,  min_en=3., max_en=70., t_step=1000, n_bins=32, orbitfile=None, nudot=0, t_ref=0):
+    """It builds the time-phase matrix
+
+    Args:
+        freq (float): folding frequency in Hertz
+        min_en (float, optional): minimum energy to accumulate the matrix. Defaults to 3..
+        max_en (float, optional): maximum energy to accumulate the matrix. Defaults to 70..
+        t_step (int, optional): time step. Defaults to 1000.
+        n_bins (int, optional): number of bins of the pulse profile. Defaults to 32.
+        orbitfile (string, optional): name of the orbit correction file. Defaults to None.
+        nudot (int, optional): frequency derivative. Defaults to 0.
+        t_ref (int, optional): folding reference time (<=0 it takes the first time). Defaults to 0 to take the first time.
+
+    Raises:
+        It writes the time phase matrixes in fits format
+    """    
     for tt in ['source', 'background']:
         for unit in ['A', 'B']:
             with open('list_evt.txt', 'w') as ff:
@@ -608,12 +626,12 @@ def make_tphase(freq,  min_en=3., max_en=70., t_step=1000, n_bins=32, orbitfile=
             with open('timing_cmd.txt', 'w') as ff:
                 if orbitfile is None:
                     ff.write(tphase_cmd % ('%s%s_TPHASE.fits' % (tt, unit),
-                                     n_bins, freq, nudot, t_step, min_en, max_en) )
+                                     n_bins, t_ref, freq, nudot, t_step, min_en, max_en) )
                 else:
                     if not os.path.isfile(orbitfile):
                         raise FileExistsError('File %s does not exist' % orbitfile)
                     ff.write(tphase_cmd_orbit % ('%s%s_TPHASE.fits' % (tt, unit), orbitfile,
-                                           n_bins, freq, nudot, t_step, min_en, max_en))
+                                           n_bins, t_ref, freq, nudot, t_step, min_en, max_en))
 
             run()
             shutil.copy('%s%s_TPHASE.fits' % (tt, unit), '%s%s_TPHASE_%03d.fits' % (tt, unit, n_bins))
@@ -741,17 +759,29 @@ def pulse_fraction_from_data_rms(counts, counts_err, n_harm=3):
     PF_rms = np.sqrt(2 * sum(bla)) / a0
     return (PF_rms)
 
-
-#this function will fit an input energy range of the pulse profile.
-#the fitting function will be a simple polynomial + gaussian
-#the aim is to retrieve basic gaussian parameters to be compared
-#to those obtained in spectral analysis around Ecycl.
-# output files: 1. the fit result
-#               2. the fit results to be plotted in a file
-#               3. Figure in pdf
-
 def get_pulsed_fraction(e_min, e_max, pp, dpp, method_calc_rms='adaptive', output_file=None, force_recompute=False):
+    """ this function fits an input energy range of the pulse profile.
+        the fitting function will be a simple polynomial + gaussian
+        the aim is to retrieve basic gaussian parameters to be compared
+        to those obtained in spectral analysis around Ecycl.
 
+    Args:
+        e_min (_type_): _description_
+        e_max (_type_): _description_
+        pp (_type_): _description_
+        dpp (_type_): _description_
+        method_calc_rms (str, optional): _description_. Defaults to 'adaptive'.
+        output_file (_type_, optional): _description_. Defaults to None.
+        force_recompute (bool, optional): _description_. Defaults to False.
+
+    Returns:
+        _type_: _description_
+        output files: 
+               1. the fit result
+               2. the fit results to be plotted in a file
+               3. Figure in pdf
+
+    """
     if output_file is not None:
         filename = output_file
         if os.path.isfile(filename) and (force_recompute is False):
@@ -791,10 +821,11 @@ def get_pulsed_fraction(e_min, e_max, pp, dpp, method_calc_rms='adaptive', outpu
     return ee_pulsed, dee_pulsed, pulsed_frac, dpulsed_frac
 
 def elaborate_pulsed_fraction(ee_pulsed, dee_pulsed, pulsed_frac, dpulsed_frac, debug_plots=True, e1=10, e2=20, 
-                              stem='', poly_deg=3, title='', save_plot=True, e_threshold=0.3, 
+                              stem='', poly_deg=[3, 3], title='', save_plot=True, e_threshold=0.3, 
                               division_derivative_order=2, max_n_high_lines = 2, forced_gaussian_centroids = [],
                               forced_gaussian_amplitudes = [],
-                              forced_gaussian_sigmas = []):
+                              forced_gaussian_sigmas = [],
+                              y_lim=[0,1]):
     """This function implements the logics to perform a fit of the pulsed fraction.
     It first interpolates the pulsed fraction with a spline to find extremes of the function.
     Then it performs a fit with a polynomial plus gaussians.
@@ -814,7 +845,7 @@ def elaborate_pulsed_fraction(ee_pulsed, dee_pulsed, pulsed_frac, dpulsed_frac,
         e2 (int, optional): lower energy to search the flex in this interval. Defaults to 20.
         Note that if you put e1>=e2, we will use only one interval.
         stem (str, optional): a string in fron of output plot names. Defaults to ''.
-        poly_deg (int, optional): degree of the polynomial. Defaults to 3.
+        poly_deg (int or list of int, optional): degree of the polynomial. Defaults to 3.
         title (str, optional): title of the plots. Defaults to ''.
         save_plot (bool, optional): if plots should be saved Defaults to True.
         e_threshold (float, optional): if energies of PF minima differ in relative terms less than this values, they are reduced.
@@ -828,10 +859,17 @@ def elaborate_pulsed_fraction(ee_pulsed, dee_pulsed, pulsed_frac, dpulsed_frac,
                                                     is different from the one of forced_gaussian_centroids, it is unused. Defaults to [].
         forced_gaussian_sigmas (list, optional): you can force the initial sigmas with this parameter, if the length of the list 
                                                  is different from the one of forced_gaussian_centroids, it is unused. Defaults to [].
+        y_lim : limits for the plotting. DEfaults to [0,1]
     Returns:
         two fitting objects for the two ranges (the output of utils.fit_pulsed_frac). if only one is used, the second is None.
      """    
     from scipy import interpolate
+
+    if type(poly_deg) is not list:
+        poly_deg = [poly_deg]
+    else:
+        poly_deg = poly_deg
+
     smoothing = interpolate.UnivariateSpline(ee_pulsed, pulsed_frac, w=1./dpulsed_frac, k=3)
     fake_en = np.linspace(ee_pulsed[0], ee_pulsed[-1], 1000)
     smoothed_pulsed_frac_deriv=smoothing.derivative(1)(fake_en)
@@ -896,25 +934,35 @@ def elaborate_pulsed_fraction(ee_pulsed, dee_pulsed, pulsed_frac, dpulsed_frac,
         ind_low = ee_pulsed <= e_turn
         ind_high = ee_pulsed > e_turn
         logger.info('We use %d points in the low-energy' % np.sum(ind_low))
+        n_gauss = 1
     else:
         #all values
         ind_low = ee_pulsed > 0
         #no value
         ind_high = ee_pulsed > 0
+        n_gauss = 1
 
         if len(forced_gaussian_centroids) > 0:
             center += [ [ee, (1-e_threshold)*ee, (1+e_threshold)*ee] for ee in forced_gaussian_centroids]
-            sigma += [[1+0*ee,0,3] for ee in forced_gaussian_centroids]
-            amplitude += [[0*ee,-2,2] for ee in forced_gaussian_centroids ]
+            if len(forced_gaussian_sigmas) == len(forced_gaussian_centroids):
+                sigma += [[ee,0,2*ee] for ee in forced_gaussian_sigmas ]
+            else:
+                sigma = [[1+0*ee,0,3] for ee in forced_gaussian_centroids]
+            
+            if len(forced_gaussian_amplitudes) == len(forced_gaussian_centroids):
+                amplitude += [[ee,-2*ee,4*ee] for ee in forced_gaussian_amplitudes ]
+            else:
+                amplitude = [[0*ee,-2,2] for ee in forced_gaussian_centroids ]
+            n_gauss = len(center)
 
 
 
     pulsed_fit_low = fit_pulsed_frac(ee_pulsed[ind_low],dee_pulsed[ind_low], 
-                                        pulsed_frac[ind_low], dpulsed_frac[ind_low], n_gauss=1, 
+                                        pulsed_frac[ind_low], dpulsed_frac[ind_low], n_gauss=n_gauss, 
                                        center=center,
                                         sigma=sigma,
                                         amplitude=amplitude,
-                                       degree_pol=poly_deg, plot_final=True, stem=stem+'low')
+                                       degree_pol=poly_deg[0], plot_final=True, stem=stem+'low', y_lim=y_lim)
 
     if make_double_fit:
 
@@ -956,7 +1004,7 @@ def elaborate_pulsed_fraction(ee_pulsed, dee_pulsed, pulsed_frac, dpulsed_frac,
             
         #print(center,sigma,amplitude)
         logger.info('We use %d points in the high-energy' % np.sum(ind_high))
-        high_poly_deg = poly_deg
+        high_poly_deg = poly_deg[-1]
         
         if np.sum(ind_high) <= high_poly_deg + n_gauss * 3:
             logger.info("Reducing polynomial order")
@@ -979,7 +1027,8 @@ def elaborate_pulsed_fraction(ee_pulsed, dee_pulsed, pulsed_frac, dpulsed_frac,
                                         center=center,
                                             sigma=sigma,
                                             amplitude=amplitude,
-                                        degree_pol=high_poly_deg, plot_final=True, stem=stem+'high')
+                                        degree_pol=high_poly_deg, plot_final=True, stem=stem+'high',
+                                        y_lim=y_lim)
     else:
         pulsed_fit_high = None
 
@@ -1008,8 +1057,8 @@ def elaborate_pulsed_fraction(ee_pulsed, dee_pulsed, pulsed_frac, dpulsed_frac,
     ax1.set_xscale('log')
     ax2.set_xlabel('E (keV)')
     ax1.set_ylabel('PF')
-    # ax1.set_ylim(ylim)
-    # ax1.set_xlim(xlim)
+    ax1.set_ylim(y_lim)
+    #ax1.set_xlim(x_lim)
     ax1.set_title(title)
 
     ax2.set_ylabel('Residuals')
@@ -1026,7 +1075,7 @@ def elaborate_pulsed_fraction(ee_pulsed, dee_pulsed, pulsed_frac, dpulsed_frac,
 
 def fit_pulsed_frac(en, den, pf, dpf, stem=None, degree_pol=4, n_gauss=0, center=[],
                     sigma=[], amplitude=[],
-                    plot_final=True, print_results=True, ylim=[0,1], xlim=[3,70]):
+                    plot_final=True, print_results=True, y_lim=[0,1], x_lim=[3,70]):
     '''
     this function will fit an input energy range of the pulse profile.
     the fitting function will be a simple polynomial + gaussian
@@ -1097,8 +1146,8 @@ def fit_pulsed_frac(en, den, pf, dpf, stem=None, degree_pol=4, n_gauss=0, center
         axes[0].legend(loc='upper left')
         axes[0].set_xlabel('E [keV]')
         axes[0].set_ylabel('PF')
-        axes[0].set_ylim(ylim)
-        axes[0].set_xlim(xlim)
+        axes[0].set_ylim(y_lim)
+        axes[0].set_xlim(x_lim)
         
 
         axes[1].errorbar(en, pf, xerr=den, yerr=dpf, fmt='.', color=col[0])
@@ -1108,8 +1157,8 @@ def fit_pulsed_frac(en, den, pf, dpf, stem=None, degree_pol=4, n_gauss=0, center
         axes[1].legend(loc='upper left')
         axes[1].set_xlabel('E [keV]')
         axes[1].set_ylabel('PF')
-        axes[1].set_ylim(ylim)
-        axes[1].set_xlim(xlim)
+        axes[1].set_ylim(y_lim)
+        axes[1].set_xlim(x_lim)
         
         plt.savefig(stem + 'fit_results.pdf')
 
@@ -1137,8 +1186,8 @@ def fit_pulsed_frac(en, den, pf, dpf, stem=None, degree_pol=4, n_gauss=0, center
         ax1.set_xscale('log')
         ax2.set_xlabel('E (keV)')
         ax1.set_ylabel('PF')
-        ax1.set_ylim(ylim)
-        ax1.set_xlim(xlim)
+        ax1.set_ylim(y_lim)
+        ax1.set_xlim(x_lim)
         
         ax2.set_ylabel('Residuals')
         ax1.set_ylim(-0.1, 1)
